@@ -69,6 +69,7 @@ import com.openexchange.file.storage.File;
 import com.openexchange.file.storage.File.Field;
 import com.openexchange.file.storage.FileDelta;
 import com.openexchange.file.storage.FileStorageAccountAccess;
+import com.openexchange.file.storage.FileStorageCaseInsensitiveAccess;
 import com.openexchange.file.storage.FileStorageExceptionCodes;
 import com.openexchange.file.storage.FileStorageFileAccess;
 import com.openexchange.file.storage.FileStorageFolderAccess;
@@ -103,7 +104,7 @@ import com.sun.mail.imap.SortTerm;
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  * @since v7.8.2
  */
-public class MailDriveFileAccess extends AbstractMailDriveResourceAccess implements FileStorageFileAccess, FileStorageSequenceNumberProvider, FileStorageReadOnly, FileStorageMailAttachments, FileStorageRangeFileAccess {
+public class MailDriveFileAccess extends AbstractMailDriveResourceAccess implements FileStorageFileAccess, FileStorageSequenceNumberProvider, FileStorageReadOnly, FileStorageMailAttachments, FileStorageRangeFileAccess, FileStorageCaseInsensitiveAccess {
 
     /** The fetch profile for a virtual folder */
     public static final FetchProfile FETCH_PROFILE_VIRTUAL = new FetchProfile() {
@@ -265,16 +266,15 @@ public class MailDriveFileAccess extends AbstractMailDriveResourceAccess impleme
         AccessControl accessControl = AccessControl.getAccessControl(session);
         try {
             accessControl.acquireGrant();
-            return getResourceReleasingStream(folderId, id, fullName);
+
+            return getResourceReleasingStream(folderId, id, fullName, accessControl);
         } catch (InterruptedException e) {
             Thread.currentThread().interrupt();
             throw MailExceptionCode.INTERRUPT_ERROR.create(e, new Object[0]);
-        } finally {
-            accessControl.close();
         }
     }
 
-    private InputStream getResourceReleasingStream(final String folderId, final String id, FullName fullName) throws OXException {
+    private InputStream getResourceReleasingStream(final String folderId, final String id, FullName fullName, AccessControl accessControl) throws OXException {
         MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> mailAccess = null;
         IMAPFolder imapFolder = null;
         boolean error = true;
@@ -290,7 +290,7 @@ public class MailDriveFileAccess extends AbstractMailDriveResourceAccess impleme
                 throw FileStorageExceptionCodes.FILE_NOT_FOUND.create(id, folderId);
             }
 
-            InputStream in = new ResourceReleasingInputStream(message.getInputStream(), imapFolder, mailAccess);
+            InputStream in = new ResourceReleasingInputStream(message.getInputStream(), imapFolder, mailAccess, accessControl);
             error = false;
             return in;
         } catch (IOException e) {
@@ -303,6 +303,7 @@ public class MailDriveFileAccess extends AbstractMailDriveResourceAccess impleme
             if (error) {
                 closeSafe(imapFolder);
                 MailAccess.closeInstance(mailAccess);
+                accessControl.close();
             }
         }
     }
@@ -827,12 +828,14 @@ public class MailDriveFileAccess extends AbstractMailDriveResourceAccess impleme
         private final InputStream in;
         private final IMAPFolder imapFolder;
         private final MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> mailAccess;
+        private final AccessControl accessControl;
 
-        ResourceReleasingInputStream(InputStream in,IMAPFolder imapFolder, MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> mailAccess) {
+        ResourceReleasingInputStream(InputStream in,IMAPFolder imapFolder, MailAccess<? extends IMailFolderStorage, ? extends IMailMessageStorage> mailAccess, AccessControl accessControl) {
             super();
             this.in = in;
             this.imapFolder = imapFolder;
             this.mailAccess = mailAccess;
+            this.accessControl = accessControl;
         }
 
         @Override
@@ -872,6 +875,7 @@ public class MailDriveFileAccess extends AbstractMailDriveResourceAccess impleme
             } finally {
                 closeSafe(imapFolder);
                 MailAccess.closeInstance(mailAccess);
+                accessControl.close();
             }
         }
 
